@@ -132,7 +132,9 @@ the default face's background color."
   (pcase theme
     ((cl-type string) theme)
     (`(,(cl-type string) . ,(cl-type string))
-     (if (color-dark-p (color-name-to-rgb (face-background 'default)))
+     (if (color-dark-p
+          (color-name-to-rgb
+           (face-background 'default (exwm--find-x-frame))))
          (cdr theme) (car theme)))
     (_ (error "Expected theme to be a string or a pair of strings"))))
 
