@@ -5,7 +5,7 @@
 ;; Author: Chris Feng <chris.w.feng@gmail.com>
 ;; Maintainer: Adrián Medraño Calvo <adrian@medranocalvo.com>, Steven Allen <steven@stebalien.com>, Daniel Mendler <mail@daniel-mendler.de>
 ;; Version: 0.34
-;; Package-Requires: ((emacs "27.1") (xelb "0.21") (compat "30"))
+;; Package-Requires: ((emacs "27.1") (xelb "0.22") (compat "30"))
 ;; Keywords: unix
 ;; URL: https://github.com/emacs-exwm/exwm
 
@@ -181,29 +181,33 @@ Set during `exwm--init'.")
   "Update _NET_WM_DESKTOP.
 Argument XWIN contains the X window of the `exwm-mode' buffer."
   (exwm--log "#x%x" xwin)
-  (with-current-buffer (exwm--id->buffer xwin)
-    (let ((reply (xcb:+request-unchecked+reply exwm--connection
-                     (make-instance 'xcb:ewmh:get-_NET_WM_DESKTOP
-                                    :window xwin)))
-          desktop)
-      (when reply
-        (setq desktop (slot-value reply 'value))
-        (cond
-         ((and desktop (= desktop 4294967295.))
-          (unless (or (not exwm--floating-frame)
-                      (eq exwm--frame exwm-workspace--current)
-                      (and exwm--desktop
-                           (= desktop exwm--desktop)))
-            (exwm-layout--show xwin (frame-root-window exwm--floating-frame)))
-          (setq exwm--desktop desktop))
-         ((and desktop
-               (< desktop (exwm-workspace--count))
-               (if exwm--desktop
-                   (/= desktop exwm--desktop)
-                 (/= desktop (exwm-workspace--position exwm--frame))))
-          (exwm-workspace-move-window desktop xwin))
-         (t
-          (exwm-workspace--set-desktop xwin)))))))
+  (with-slots (value)
+      (xcb:+request-unchecked+reply exwm--connection
+          (make-instance 'xcb:ewmh:get-_NET_WM_DESKTOP
+                         :window xwin))
+    (exwm--set-desktop xwin value)))
+
+(defun exwm--set-desktop (id desktop)
+  "Set ID's workspace to DESKTOP."
+  (when-let* ((buffer (exwm--id->buffer id))
+              (_(buffer-live-p buffer)))
+    (with-current-buffer buffer
+      (cond
+       ((and desktop (= desktop #xffffffff))
+        (unless (or (not exwm--floating-frame)
+                    (eq exwm--frame exwm-workspace--current)
+                    (and exwm--desktop
+                         (= desktop exwm--desktop)))
+          (exwm-layout--show id (frame-root-window exwm--floating-frame)))
+        (setq exwm--desktop desktop))
+       ((and desktop
+             (< desktop (exwm-workspace--count))
+             (if exwm--desktop
+                 (/= desktop exwm--desktop)
+               (/= desktop (exwm-workspace--position exwm--frame))))
+        (exwm-workspace-move-window desktop id))
+       (t
+        (exwm-workspace--set-desktop id))))))
 
 (defun exwm--update-window-type (id &optional force)
   "Update `exwm-window-type' from _NET_WM_WINDOW_TYPE.
@@ -554,13 +558,11 @@ Descriptors' for the list of supported properties."
 (defun exwm--on-PropertyNotify (data _synthetic)
   "Handle PropertyNotify event.
 DATA contains unmarshalled PropertyNotify event data."
-  (let ((obj (make-instance 'xcb:PropertyNotify))
-        atom id buffer)
-    (xcb:unmarshal obj data)
-    (setq id (slot-value obj 'window)
-          atom (slot-value obj 'atom))
+  (let* ((obj (xcb:unmarshal-new 'xcb:PropertyNotify data))
+         (atom (slot-value obj 'atom))
+         (id (slot-value obj 'window))
+         (buffer (exwm--id->buffer id)))
     (exwm--log "atom=%s(%s)" (x-get-atom-name atom exwm-workspace--current) atom)
-    (setq buffer (exwm--id->buffer id))
     (if (not (buffer-live-p buffer))
         ;; Properties of unmanaged X windows.
         (cond ((= atom xcb:Atom:_NET_WM_STRUT)
@@ -702,9 +704,7 @@ DATA contains unmarshalled PropertyNotify event data."
 
 (defun exwm--on-net-wm-desktop (id data)
   "Handle _NET_WM_DESKTOP message with ID and DATA."
-  (let ((buffer (exwm--id->buffer id)))
-    (when (buffer-live-p buffer)
-      (exwm-workspace-move-window (elt data 0) id))))
+  (exwm--set-desktop id (elt data 0)))
 
 (defun exwm--on-net-wm-state (id data)
   "Handle _NET_WM_STATE message with ID and DATA."
@@ -782,9 +782,7 @@ DATA contains unmarshalled PropertyNotify event data."
 (defun exwm--on-ClientMessage (raw-data _synthetic)
   "Handle ClientMessage event.
 RAW-DATA contains unmarshalled ClientMessage event data."
-  (let* ((obj (let ((m (make-instance 'xcb:ClientMessage)))
-                (xcb:unmarshal m raw-data)
-                m))
+  (let* ((obj (xcb:unmarshal-new 'xcb:ClientMessage raw-data))
          (type (slot-value obj 'type))
          (id (slot-value obj 'window))
          (data (slot-value (slot-value obj 'data) 'data32))
@@ -801,11 +799,8 @@ RAW-DATA contains unmarshalled ClientMessage event data."
   "Handle SelectionClear events.
 DATA contains unmarshalled SelectionClear event data."
   (exwm--log)
-  (let ((obj (make-instance 'xcb:SelectionClear))
-        owner selection)
-    (xcb:unmarshal obj data)
-    (setq owner (slot-value obj 'owner)
-          selection (slot-value obj 'selection))
+  (with-slots (owner selection)
+      (xcb:unmarshal-new 'xcb:SelectionClear data)
     (when (and (eq owner exwm--wmsn-window)
                (eq selection xcb:Atom:WM_S0))
       (exwm-wm-mode -1))))
@@ -1148,13 +1143,6 @@ FRAME, if given, indicates the X display EXWM should manage."
       (exwm--exit))
     (exwm--disable)))
 
-(defun exwm--find-x-frame ()
-  "Find a frame whose terminal is an X display.
-Selected frame is checked first."
-  (cl-loop for term in (cons (frame-terminal (selected-frame)) (terminal-list))
-           if (eq 'x (terminal-live-p term))
-           return (car (frames-on-display-list term))))
-
 (defun exwm--disable ()
   "Unregister functions for EXWM to be initialized."
   (exwm--log)
